@@ -56,6 +56,7 @@
 
 #include "lzx.h"
 
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #ifdef CHM_DEBUG
@@ -66,13 +67,6 @@
 #include <strings.h>
 #endif
 
-/* RWE 6/13/2003 */
-#ifdef _WIN32_WCE
-#define FREEBUF(x) free(x)
-#else
-#define FREEBUF(x) /* do nothing */
-#endif
-
 #ifdef WIN32
 #include <windows.h>
 #include <malloc.h>
@@ -156,22 +150,9 @@ typedef unsigned __int32        UInt32;
 typedef __int64                 Int64;
 typedef unsigned __int64        UInt64;
 
-/* I386, 32-bit, non-Windows */
-/* Sparc        */
-/* MIPS         */
-/* PPC          */
-#elif __i386__ || __sun || __sgi || __ppc__
-typedef unsigned char           UChar;
-typedef short                   Int16;
-typedef unsigned short          UInt16;
-typedef long                    Int32;
-typedef unsigned long           UInt32;
-typedef long long               Int64;
-typedef unsigned long long      UInt64;
-
 /* x86-64 */
 /* Note that these may be appropriate for other 64-bit machines. */
-#elif __x86_64__ || __ia64__
+#elif defined(__LP64__)
 typedef unsigned char           UChar;
 typedef short                   Int16;
 typedef unsigned short          UInt16;
@@ -180,10 +161,18 @@ typedef unsigned int            UInt32;
 typedef long                    Int64;
 typedef unsigned long           UInt64;
 
+/* I386, 32-bit, non-Windows */
+/* Sparc        */
+/* MIPS         */
+/* PPC          */
 #else
-
-/* yielding an error is preferable to yielding incorrect behavior */
-#error "Please define the sized types for your platform in chm_lib.c"
+typedef unsigned char           UChar;
+typedef short                   Int16;
+typedef unsigned short          UInt16;
+typedef long                    Int32;
+typedef unsigned long           UInt32;
+typedef long long               Int64;
+typedef unsigned long long      UInt64;
 #endif
 
 /* GCC */
@@ -775,6 +764,8 @@ struct chmFile *chm_open(const char *filename)
 
     /* allocate handle */
     newHandle = (struct chmFile *)malloc(sizeof(struct chmFile));
+    if (newHandle == NULL)
+        return NULL;
     newHandle->fd = CHM_NULL_FD;
     newHandle->lzx_state = NULL;
     newHandle->cache_blocks = NULL;
@@ -1041,7 +1032,9 @@ void chm_set_param(struct chmFile *h,
 
                 /* allocate new cached blocks */
                 newBlocks = (UChar **)malloc(paramVal * sizeof (UChar *));
+                if (newBlocks == NULL) return;
                 newIndices = (UInt64 *)malloc(paramVal * sizeof (UInt64));
+                if (newIndices == NULL) { free(newBlocks); return; }
                 for (i=0; i<paramVal; i++)
                 {
                     newBlocks[i] = NULL;
@@ -1262,16 +1255,10 @@ int chm_resolve_object(struct chmFile *h,
     Int32 curPage;
 
     /* buffer to hold whatever page we're looking at */
-#ifdef WIN32
-#ifdef _WIN32_WCE
     /* RWE 6/12/2003 */
     UChar *page_buf = malloc(h->block_len);
-#else
-    UChar *page_buf = alloca(h->block_len);
-#endif
-#else
-    UChar page_buf[h->block_len];
-#endif
+    if (page_buf == NULL)
+        return CHM_RESOLVE_FAILURE;
 
     /* starting page */
     curPage = h->index_root;
@@ -1285,7 +1272,7 @@ int chm_resolve_object(struct chmFile *h,
                              (UInt64)h->dir_offset + (UInt64)curPage*h->block_len,
                              h->block_len) != h->block_len)
         {
-            FREEBUF(page_buf);
+            free(page_buf);
             return CHM_RESOLVE_FAILURE;
         }
 
@@ -1298,13 +1285,13 @@ int chm_resolve_object(struct chmFile *h,
                                               objPath);
             if (pEntry == NULL)
             {
-                FREEBUF(page_buf);
+                free(page_buf);
                 return CHM_RESOLVE_FAILURE;
             }
 
             /* parse entry and return */
             _chm_parse_PMGL_entry(&pEntry, ui);
-            FREEBUF(page_buf);
+            free(page_buf);
             return CHM_RESOLVE_SUCCESS;
         }
 
@@ -1315,13 +1302,13 @@ int chm_resolve_object(struct chmFile *h,
         /* else, we are confused.  give up. */
         else
         {
-            FREEBUF(page_buf);
+            free(page_buf);
             return CHM_RESOLVE_FAILURE;
         }
     }
 
     /* didn't find anything.  fail. */
-    FREEBUF(page_buf);
+    free(page_buf);
     return CHM_RESOLVE_FAILURE;
 }
 
@@ -1396,16 +1383,7 @@ static Int64 _chm_decompress_block(struct chmFile *h,
                                    UInt64 block,
                                    UChar **ubuffer)
 {
-#ifdef WIN32
-#ifdef _WIN32_WCE
-    /* RWE 6/12/2003 */
     UChar *cbuffer = malloc(((unsigned int)h->reset_table.block_len + 6144));
-#else
-    UChar *cbuffer = alloca(((unsigned int)h->reset_table.block_len + 6144));
-#endif
-#else
-    UChar cbuffer[h->reset_table.block_len + 6144];     /* compressed buffer */
-#endif
     UInt64 cmpStart;                                    /* compressed start  */
     Int64 cmpLen;                                       /* compressed len    */
     int indexSlot;                                      /* cache index slot  */
@@ -1413,6 +1391,9 @@ static Int64 _chm_decompress_block(struct chmFile *h,
     UInt32 blockAlign = (UInt32)(block % h->reset_blkcount); /* reset intvl. aln. */
     UInt32 i;                                           /* local loop index  */
 
+    if (cbuffer == NULL)
+        return -1;
+
     /* let the caching system pull its weight! */
     if (block - blockAlign <= h->lzx_last_block  &&
         block              >= h->lzx_last_block)
@@ -1438,10 +1419,14 @@ static Int64 _chm_decompress_block(struct chmFile *h,
                 }
 
                 indexSlot = (int)((curBlockIdx) % h->cache_num_blocks);
-                h->cache_block_indices[indexSlot] = curBlockIdx;
                 if (! h->cache_blocks[indexSlot])
-                    h->cache_blocks[indexSlot] = (UChar *)malloc(
-                                                                 (unsigned int)(h->reset_table.block_len));
+                    h->cache_blocks[indexSlot] = (UChar *)malloc((unsigned int)(h->reset_table.block_len));
+                if (! h->cache_blocks[indexSlot])
+                {
+                    free(cbuffer);
+                    return -1;
+                }
+                h->cache_block_indices[indexSlot] = curBlockIdx;
                 lbuffer = h->cache_blocks[indexSlot];
 
                 /* decompress the previous block */
@@ -1458,7 +1443,7 @@ static Int64 _chm_decompress_block(struct chmFile *h,
 #ifdef CHM_DEBUG
                     fprintf(stderr, "   (DECOMPRESS FAILED!)\n");
 #endif
-                    FREEBUF(cbuffer);
+                    free(cbuffer);
                     return (Int64)0;
                 }
 
@@ -1479,10 +1464,14 @@ static Int64 _chm_decompress_block(struct chmFile *h,
 
     /* allocate slot in cache */
     indexSlot = (int)(block % h->cache_num_blocks);
-    h->cache_block_indices[indexSlot] = block;
     if (! h->cache_blocks[indexSlot])
-        h->cache_blocks[indexSlot] = (UChar *)malloc(
-                                          ((unsigned int)h->reset_table.block_len));
+        h->cache_blocks[indexSlot] = (UChar *)malloc(((unsigned int)h->reset_table.block_len));
+    if (! h->cache_blocks[indexSlot])
+    {
+        free(cbuffer);
+        return -1;
+    }
+    h->cache_block_indices[indexSlot] = block;
     lbuffer = h->cache_blocks[indexSlot];
     *ubuffer = lbuffer;
 
@@ -1498,7 +1487,7 @@ static Int64 _chm_decompress_block(struct chmFile *h,
 #ifdef CHM_DEBUG
         fprintf(stderr, "   (DECOMPRESS FAILED!)\n");
 #endif
-        FREEBUF(cbuffer);
+        free(cbuffer);
         return (Int64)0;
     }
     h->lzx_last_block = (int)block;
@@ -1506,7 +1495,7 @@ static Int64 _chm_decompress_block(struct chmFile *h,
     /* XXX: modify LZX routines to return the length of the data they
      * decompressed and return that instead, for an extra sanity check.
      */
-    FREEBUF(cbuffer);
+    free(cbuffer);
     return h->reset_table.block_len;
 }
 
@@ -1631,16 +1620,8 @@ int chm_enumerate(struct chmFile *h,
     Int32 curPage;
 
     /* buffer to hold whatever page we're looking at */
-#ifdef WIN32
-#ifdef _WIN32_WCE
     /* RWE 6/12/2003 */
     UChar *page_buf = malloc((unsigned int)h->block_len);
-#else
-    UChar *page_buf = alloca((unsigned int)h->block_len);
-#endif
-#else
-    UChar page_buf[h->block_len];
-#endif
     struct chmPmglHeader header;
     UChar *end;
     UChar *cur;
@@ -1652,6 +1633,9 @@ int chm_enumerate(struct chmFile *h,
     int type_bits = (what & 0x7);
     int filter_bits = (what & 0xF8);
 
+    if (page_buf == NULL)
+        return 0;
+
     /* starting page */
     curPage = h->index_head;
 
@@ -1665,7 +1649,7 @@ int chm_enumerate(struct chmFile *h,
                              (UInt64)h->dir_offset + (UInt64)curPage*h->block_len,
                              h->block_len) != h->block_len)
         {
-            FREEBUF(page_buf);
+            free(page_buf);
             return 0;
         }
 
@@ -1674,7 +1658,7 @@ int chm_enumerate(struct chmFile *h,
         lenRemain = _CHM_PMGL_LEN;
         if (! _unmarshal_pmgl_header(&cur, &lenRemain, &header))
         {
-            FREEBUF(page_buf);
+            free(page_buf);
             return 0;
         }
         end = page_buf + h->block_len - (header.free_space);
@@ -1686,7 +1670,7 @@ int chm_enumerate(struct chmFile *h,
 
             if (! _chm_parse_PMGL_entry(&cur, &ui))
             {
-                FREEBUF(page_buf);
+                free(page_buf);
                 return 0;
             }
 
@@ -1726,12 +1710,12 @@ int chm_enumerate(struct chmFile *h,
                 switch (status)
                 {
                     case CHM_ENUMERATOR_FAILURE:
-                        FREEBUF(page_buf);
+                        free(page_buf);
                         return 0;
                     case CHM_ENUMERATOR_CONTINUE:
                         break;
                     case CHM_ENUMERATOR_SUCCESS:
-                        FREEBUF(page_buf);
+                        free(page_buf);
                         return 1;
                     default:
                         break;
@@ -1743,7 +1727,7 @@ int chm_enumerate(struct chmFile *h,
         curPage = header.block_next;
     }
 
-    FREEBUF(page_buf);
+    free(page_buf);
     return 1;
 }
 
@@ -1760,16 +1744,8 @@ int chm_enumerate_dir(struct chmFile *h,
     Int32 curPage;
 
     /* buffer to hold whatever page we're looking at */
-#ifdef WIN32
-#ifdef _WIN32_WCE
     /* RWE 6/12/2003 */
     UChar *page_buf = malloc((unsigned int)h->block_len);
-#else
-    UChar *page_buf = alloca((unsigned int)h->block_len);
-#endif
-#else
-    UChar page_buf[h->block_len];
-#endif
     struct chmPmglHeader header;
     UChar *end;
     UChar *cur;
@@ -1790,6 +1766,9 @@ int chm_enumerate_dir(struct chmFile *h,
     char lastPath[CHM_MAX_PATHLEN+1];
     int lastPathLen;
 
+    if (page_buf == NULL)
+        return 0;
+
     /* starting page */
     curPage = h->index_head;
 
@@ -1819,7 +1798,7 @@ int chm_enumerate_dir(struct chmFile *h,
                              (UInt64)h->dir_offset + (UInt64)curPage*h->block_len,
                              h->block_len) != h->block_len)
         {
-            FREEBUF(page_buf);
+            free(page_buf);
             return 0;
         }
 
@@ -1828,7 +1807,7 @@ int chm_enumerate_dir(struct chmFile *h,
         lenRemain = _CHM_PMGL_LEN;
         if (! _unmarshal_pmgl_header(&cur, &lenRemain, &header))
         {
-            FREEBUF(page_buf);
+            free(page_buf);
             return 0;
         }
         end = page_buf + h->block_len - (header.free_space);
@@ -1840,7 +1819,7 @@ int chm_enumerate_dir(struct chmFile *h,
 
             if (! _chm_parse_PMGL_entry(&cur, &ui))
             {
-                FREEBUF(page_buf);
+                free(page_buf);
                 return 0;
             }
 
@@ -1861,7 +1840,7 @@ int chm_enumerate_dir(struct chmFile *h,
             {
                 if (strncasecmp(ui.path, prefixRectified, prefixLen) != 0)
                 {
-                    FREEBUF(page_buf);
+                    free(page_buf);
                     return 1;
                 }
             }
@@ -1912,12 +1891,12 @@ int chm_enumerate_dir(struct chmFile *h,
                 switch (status)
                 {
                     case CHM_ENUMERATOR_FAILURE:
-                        FREEBUF(page_buf);
+                        free(page_buf);
                         return 0;
                     case CHM_ENUMERATOR_CONTINUE:
                         break;
                     case CHM_ENUMERATOR_SUCCESS:
-                        FREEBUF(page_buf);
+                        free(page_buf);
                         return 1;
                     default:
                         break;
@@ -1929,6 +1908,6 @@ int chm_enumerate_dir(struct chmFile *h,
         curPage = header.block_next;
     }
 
-    FREEBUF(page_buf);
+    free(page_buf);
     return 1;
 }
